import os
import tempfile
import unittest

from ..scaffolding import read_csv_and_write_output
from ..helpers import project_dir
from .data import LIST_GENERATED_CURRENCY_MODULES

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("TestScaffolding",)


class TestScaffolding(unittest.TestCase):
    """Scaffolding tests."""

    def test_generate_currencies(self):
        """Test generate currency modules."""
        out_dir = tempfile.mkdtemp()
        read_csv_and_write_output(out_dir=out_dir, skip_first_line=True)
        generated_modules = os.listdir(out_dir)
        generated_modules.sort()
        self.assertListEqual(
            LIST_GENERATED_CURRENCY_MODULES, generated_modules
        )

    def test_generate_currencies_rel_path(self):
        """Test generate currency modules relative paths."""
        os.makedirs(project_dir("var"), exist_ok=True)
        out_dir = read_csv_and_write_output(
            out_dir="var", skip_first_line=True
        )
        generated_modules = os.listdir(out_dir)
        generated_modules.sort()
        self.assertListEqual(
            LIST_GENERATED_CURRENCY_MODULES, generated_modules
        )



from typing import Tuple, List, Set, Union
from .base import Registry

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = (
    "get_currency_choices",
    "get_currency_choices_with_code",
)


def get_currency_choices(
    limit_choices_to: Union[Tuple[str, ...], List[str], Set[str]] = None,
    sort_by_key: bool = False,
) -> List[Tuple[str, str]]:
    """Get currency choices.

    List of choices in the following format::

        [
            ("AMD", "Armenian Dram"),
            ("EUR", "Euro"),
        ]
    """
    return Registry.values(limit_choices_to, sort_by_key)


def get_currency_choices_with_code(
    limit_choices_to: Union[Tuple[str, ...], List[str], Set[str]] = None,
    sort_by_key: bool = False,
) -> List[Tuple[str, str]]:
    """Get currency choices with code.

    List of choices in the following format::

        [
            ("AMD", "Armenian Dram (AMD)"),
            ("EUR", "Euro (EUR)"),
        ]
    """
    return Registry.values_with_code(limit_choices_to, sort_by_key)



from ..base import BaseCurrency

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("MVR",)


class MVR(BaseCurrency):
    """MVR - Maldivian rufiyaa."""

    uid: str = "MVR"
    rate: int = 100



import unittest

from ..currencies import TND, EUR, MRU, VND, UGX
from ..exceptions import InvalidCurrency
from ..shortcuts import convert_to_currency_units


__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("TestShortcuts",)


class TestShortcuts(unittest.TestCase):
    """Test shortcuts."""

    def test_convert_to_currency_units(self):
        """Test `convert_to_currency_units` with valid codes."""
        self.assertEqual(convert_to_currency_units(UGX.uid, 10), 10)

        self.assertEqual(convert_to_currency_units(MRU.uid, 5), 1)

        self.assertEqual(convert_to_currency_units(VND.uid, 10), 1)

        self.assertEqual(convert_to_currency_units(EUR.uid, 100), 1)

        self.assertEqual(convert_to_currency_units(TND.uid, 1_000), 1)

    def _test_currency_code_is_none(self, fail_silently: bool = True):
        self.assertEqual(
            convert_to_currency_units(None, 1_000, fail_silently), None
        )

    def _test_currency_code_is_empty_string(self, fail_silently: bool = True):
        self.assertEqual(
            convert_to_currency_units("", 1_000, fail_silently), None
        )

    def _test_currency_code_is_invalid(self, fail_silently: bool = True):
        self.assertEqual(
            convert_to_currency_units("None", 1_000, fail_silently), None
        )

    def test_test_convert_to_currency_units_invalid_currency_code(self):
        """Test `convert_to_currency_units` with invalid codes."""

        with self.subTest("currency_code=None"):
            self._test_currency_code_is_none()

        with self.subTest("currency_code=''"):
            self._test_currency_code_is_empty_string()

        with self.subTest("currency_code='None'"):
            self._test_currency_code_is_invalid()

        with self.assertRaises(InvalidCurrency):
            with self.subTest("currency_code=None"):
                self._test_currency_code_is_none(fail_silently=False)

        with self.assertRaises(InvalidCurrency):
            with self.subTest("currency_code=''"):
                self._test_currency_code_is_empty_string(fail_silently=False)

        with self.assertRaises(InvalidCurrency):
            with self.subTest("currency_code='None'"):
                self._test_currency_code_is_invalid(fail_silently=False)



from ..base import BaseCurrency

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("KPW",)


class KPW(BaseCurrency):
    """KPW - North Korean won."""

    uid: str = "KPW"
    rate: int = 100



__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = (
    "ImproperlyConfigured",
    "InvalidCurrency",
)


class ImproperlyConfigured(Exception):
    """ImproperlyConfigured.

    Supposed to be thrown when code is improperly configured.
    """


class InvalidCurrency(ValueError):
    """Invalid currency exception."""



from importlib import import_module
from inspect import isclass
import os
from pkgutil import iter_modules

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"


# Iterate through the modules in the current package
package_dir = os.path.dirname(__file__)

__all = []

for (_, module_name, _) in iter_modules([package_dir]):

    # Import the module and iterate through its attributes
    module = import_module(f"{__name__}.{module_name}")
    __all.extend(module.__all__)
    for attribute_name in dir(module):
        attribute = getattr(module, attribute_name)

        if isclass(attribute):
            # Add the class to this package's variables
            globals()[attribute_name] = attribute

__all__ = tuple(__all)



from copy import deepcopy
import unittest

from ..base import BaseCurrency, Registry
from ..currencies import TND, EUR, MRU, VND, UGX
from ..exceptions import ImproperlyConfigured
from ..utils import get_currency_choices_with_code

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("TestBase",)


class TestBase(unittest.TestCase):
    """Base tests."""

    def setUp(self) -> None:
        self.registry_backup = deepcopy(Registry.REGISTRY)

    def tearDown(self) -> None:
        Registry.REGISTRY = self.registry_backup

    def test_subclass(self):
        """Test subclass."""

        class XYZ(BaseCurrency):
            """XYZ - Test currency."""

            uid: str = "XYZ"
            rate: int = 100_000_000

        self.assertIn(XYZ.uid, Registry.REGISTRY)
        self.assertEqual(XYZ.convert_to_currency_units(100_000_000), 1)

    def test_subclass_no_rate(self):
        """Test subclass no `rate`."""

        with self.assertRaises(ImproperlyConfigured):

            class CurrencyNoRate(BaseCurrency):
                uid: str = "XYZ"

    def test_symbol(self):
        """Test symbol."""
        self.assertEqual(Registry.get("EUR").symbol, "€")

    def test_1(self):
        self.assertEqual(UGX.convert_to_currency_units(10), 10)
        self.assertEqual(UGX.convert_to_currency_units(1_000), 1_000)

    def test_5(self):
        self.assertEqual(MRU.convert_to_currency_units(5), 1)
        self.assertEqual(MRU.convert_to_currency_units(1_000), 200)

    def test_10(self):
        self.assertEqual(VND.convert_to_currency_units(10), 1)
        self.assertEqual(VND.convert_to_currency_units(1_000), 100)

    def test_100(self):
        self.assertEqual(EUR.convert_to_currency_units(100), 1)
        self.assertEqual(EUR.convert_to_currency_units(1_000), 10)

    def test_1000(self):
        self.assertEqual(TND.convert_to_currency_units(1_000), 1)


class TestRegistry(unittest.TestCase):
    """Test registry."""

    def setUp(self) -> None:
        self.registry_backup = deepcopy(Registry.REGISTRY)

    def tearDown(self) -> None:
        Registry.REGISTRY = self.registry_backup

    def test_get_currency_choices(self):
        """Test Registry.reset."""
        Registry.reset()
        choices = get_currency_choices_with_code()
        self.assertListEqual(choices, [])

    def test_registry_items(self):
        items = Registry.items()
        choices = get_currency_choices_with_code()
        self.assertEqual(len(items), len(choices))



import os

from setuptools import setup, find_packages

version = "0.1.7"

try:
    readme = open(os.path.join(os.path.dirname(__file__), "README.rst")).read()
except:
    readme = ""

template_dirs = []
static_dirs = []
locale_dirs = []

templates = []
static_files = []
locale_files = []

for template_dir in template_dirs:
    templates += [
        os.path.join(template_dir, f) for f in os.listdir(template_dir)
    ]

for static_dir in static_dirs:
    static_files += [
        os.path.join(static_dir, f) for f in os.listdir(static_dir)
    ]

for locale_dir in locale_dirs:
    locale_files += [
        os.path.join(locale_dir, f) for f in os.listdir(locale_dir)
    ]

dependency_links = []

# Dependencies
install_requires = [
    "Babel>=2.1.1",
]

tests_require = [
    "Faker",
    "pytest",
    "pytest-cov",
    "pytest-django",
    "pytest-ordering",
    "coverage",
]

setup(
    name="valuta",
    version=version,
    description="Currencies done right.",
    long_description=f"{readme}",
    long_description_content_type="text/x-rst",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/barseghyanartur/valuta/issues",
        "Documentation": "https://valuta.readthedocs.io/",
        "Source Code": "https://github.com/barseghyanartur/valuta",
        "Changelog": "https://valuta.readthedocs.io/"
        "en/latest/changelog.html",
    },
    keywords="currency",
    author="Artur Barseghyan",
    author_email="artur.barseghyan@gmail.com",
    url="https://github.com/barseghyanartur/valuta/",
    package_dir={"": "src"},
    packages=find_packages(where="./src"),
    entry_points={
        "console_scripts": [
            "valuta-generate-currencies = valuta.cli:generate_currencies",
            "valuta-list-currencies = valuta.cli:list_currencies",
        ]
    },
    license="GPL-2.0-only OR LGPL-2.1-or-later",
    install_requires=install_requires,
    tests_require=tests_require,
    dependency_links=dependency_links,
    package_data={"valuta": templates + static_files + locale_files},
    include_package_data=True,
)



from decimal import Decimal
import unittest

from django.utils.translation import gettext_lazy as _
from django.test import TestCase, override_settings

import valuta
from valuta.contrib.django_integration.models import CurrencyField
from valuta.utils import get_currency_choices

from product.models import (
    AbstractProduct,
    Product,
    ProductProxyCastToInt,
    ProductProxyCastToFloat,
    ProductProxyCastToDecimal,
    ProductProxyLimitChoicesTo,
    ProductProxyChoicesFuncNone,
)

__author__ = "Artur Barseghyan"
__copyright__ = "2021 Artur Barseghyan"
__license__ = "GPL-2.0-only OR LGPL-2.1-or-later"
__all__ = ("DjangoIntegrationTestCase",)


class DjangoIntegrationTestCase(TestCase):
    def test_limit_choices_to(self):
        with self.subTest("Field choices"):
            self.assertListEqual(
                ProductProxyLimitChoicesTo.currency.field.choices,
                [("AMD", "Armenian Dram (AMD)"), ("EUR", "Euro (EUR)")],
            )

    @override_settings(
        VALUTA_FIELD_LIMIT_CHOICES_TO=(
            valuta.AMD.uid,
            valuta.EUR.uid,
        ),
    )
    def test_limit_choices_to_from_django_settings(self):
        class ProductValueFieldLimitChoicesSettings(AbstractProduct):
            """Product model."""

            currency = CurrencyField(
                _("Currency"),
                amount_fields=(
                    "price",
                    "price_with_tax",
                ),
                get_choices_func=get_currency_choices,
            )

            class Meta:
                managed = False
                app_label = "product"
                db_table = Product._meta.db_table
                verbose_name = _(
                    "Product proxy (VALUTA_FIELD_LIMIT_CHOICES_TO)"
                )
                verbose_name_plural = _(
                    "Product proxies (VALUTA_FIELD_LIMIT_CHOICES_TO)"
                )

        with self.subTest("Field choices from django settings"):
            self.assertListEqual(
                ProductValueFieldLimitChoicesSettings.currency.field.choices,
                [("AMD", "Armenian Dram"), ("EUR", "Euro")],
            )

    def test_no_cast(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertEqual(price_in_currency_units, 100)

        price_with_tax_in_currency_units = p.price_with_tax_in_currency_units()
        self.assertEqual(price_with_tax_in_currency_units, 120)

    def test_cast_to_int(self):
        p = ProductProxyCastToInt.objects.create(
            **{
                "name": "My test product",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertEqual(price_in_currency_units, 100)
        self.assertTrue(isinstance(price_in_currency_units, int))

        price_with_tax_in_currency_units = p.price_with_tax_in_currency_units()
        self.assertEqual(price_with_tax_in_currency_units, 120)
        self.assertTrue(isinstance(price_in_currency_units, int))

    def test_cast_to_float(self):
        p = ProductProxyCastToFloat.objects.create(
            **{
                "name": "My test product",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertEqual(price_in_currency_units, 100.0)
        self.assertTrue(isinstance(price_in_currency_units, float))

        price_with_tax_in_currency_units = p.price_with_tax_in_currency_units()
        self.assertEqual(price_with_tax_in_currency_units, 120.0)
        self.assertTrue(isinstance(price_in_currency_units, float))

    def test_cast_to_decimal(self):
        p = ProductProxyCastToDecimal.objects.create(
            **{
                "name": "My test product",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertEqual(price_in_currency_units, Decimal("100.0"))
        self.assertTrue(isinstance(price_in_currency_units, Decimal))

        price_with_tax_in_currency_units = p.price_with_tax_in_currency_units()
        self.assertEqual(price_with_tax_in_currency_units, Decimal("120.0"))
        self.assertTrue(isinstance(price_in_currency_units, Decimal))

    def test_empty_currency_value(self):
        p = Product.objects.create(
            **{
                "name": "My test product 2",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": "",
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertIsNone(price_in_currency_units)

    def test_invalid_currency_value(self):
        p = Product.objects.create(
            **{
                "name": "My test product 2",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": "INVALID",
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertIsNone(price_in_currency_units)

    def test_get_choices_func_is_none(self):
        p = ProductProxyChoicesFuncNone.objects.create(
            **{
                "name": "My test product 3",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        price_in_currency_units = p.price_in_currency_units()
        self.assertEqual(price_in_currency_units, 100)

        price_with_tax_in_currency_units = p.price_with_tax_in_currency_units()
        self.assertEqual(price_with_tax_in_currency_units, 120)

    def test_get_currency_cls(self):
        p = ProductProxyChoicesFuncNone.objects.create(
            **{
                "name": "My test product 4",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": valuta.EUR.uid,
            }
        )
        currency_cls = p.get_currency_cls_for_currency()
        self.assertEqual(currency_cls, valuta.EUR)

    def test_get_currency_cls_empty_currency_value(self):
        p = ProductProxyChoicesFuncNone.objects.create(
            **{
                "name": "My test product 5",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": "",
            }
        )
        currency_cls = p.get_currency_cls_for_currency()
        self.assertIsNone(currency_cls)

    def test_get_currency_cls_invalid_currency_value(self):
        p = ProductProxyChoicesFuncNone.objects.create(
            **{
                "name": "My test product 6",
                "price": 10000,
                "price_with_tax": 12000,
                "currency": "INVALID",
            }
        )
        currency_cls = p.get_currency_cls_for_currency()
        self.assertIsNone(currency_cls)

    def test_1(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 1_000,
                "price_with_tax": 1_200,
                "currency": valuta.UGX.uid,
            }
        )
        self.assertEqual(p.price_in_currency_units(), 1_000)

    def test_5(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 1_000,
                "price_with_tax": 1_200,
                "currency": valuta.MRU.uid,
            }
        )
        self.assertEqual(p.price_in_currency_units(), 200)

    def test_10(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 1_000,
                "price_with_tax": 1_200,
                "currency": valuta.VND.uid,
            }
        )
        self.assertEqual(p.price_in_currency_units(), 100)

    def test_100(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 1_000,
                "price_with_tax": 1_200,
                "currency": valuta.EUR.uid,
            }
        )
        self.assertEqual(p.price_in_currency_units(), 10)

    def test_1000(self):
        p = Product.objects.create(
            **{
                "name": "My test product",
                "price": 1_000,
                "price_with_tax": 1_200,
                "currency": valuta.TND.uid,
            }
        )
        self.assertEqual(p.price_in_currency_units(), 1)
